require "Reloading/ISReloadableWeapon"

ZXBCrossbowWeapon = ISReloadableWeapon:derive("ZXBCrossbowWeapon");

--************************************************************************--
--** ZXBCrossbowWeapon:initialise
--**
--************************************************************************--
function ZXBCrossbowWeapon:initialise()

end

--************************************************************************--
--** ZXBCrossbowWeapon:new
--**
--** Constructor for the ZXBCrossbowWeapon reloadable
--************************************************************************--
function ZXBCrossbowWeapon:new()
	local o = {}
	--o.data = {}
	o = ISReloadableWeapon:new();
    setmetatable(o, self)
    self.__index = self
	return o;
end

--************************************************************************--
--** ZXBCrossbowWeapon:isLoaded
--**
--** Returns whether or not the gun will fire when the mouse is next
--** clicked
--**
--************************************************************************--
function ZXBCrossbowWeapon:isLoaded(difficulty)
	return self.roundChambered == 1;
end

--************************************************************************--
--** ZXBCrossbowWeapon:fireShot
--**
--** Action performed when a shot is fired. Should typically decrease
--** the current amount of ammo in the weapon
--**
--************************************************************************--
function ZXBCrossbowWeapon:fireShot(weapon, difficulty)	
	self.roundChambered = 0;
	self.currentCapacity = 0;
	--self.emptyShellChambered = 1;	 
	self:syncReloadableToItem(weapon);
end

--************************************************************************--
--** ZXBCrossbowWeapon:canReload
--**
--** Whether the character attempting to reload has the necessary
--** prerequisites to perform the reload action. Called prior to
--** the timed action and not to be confused with isReloadValid
--**
--************************************************************************--
function ZXBCrossbowWeapon:canReload(chr)
	if(self.currentCapacity < self.maxCapacity
	and (chr:getInventory():FindAndReturn(self.ammoType) ~= nil or self:hasBoltsInPouch(chr))) then		
		return true;
	end
	return false;
end
function ZXBCrossbowWeapon:canRack(chr)	
	return false;
end
--************************************************************************--
--** ZXBCrossbowWeapon:isReloadValid
--**
--** Function for the TimedAction that determines whether the reload
--** action is still valid. If the player does something that should
--** interrupt the action, this should return false
--**
--** @param char - the character performing the action. Must not be nil
--** @param square - not used
--**
--** @return true if the action may continue to be performed
--**
--************************************************************************--
function ZXBCrossbowWeapon:isReloadValid(char, square, difficulty)	

	if(self.currentCapacity < self.maxCapacity
		and (char:getInventory():FindAndReturn(self.ammoType) ~= nil or self:hasBoltsInPouch(char))) then
			return true;
	end
	self.reloadInProgress = false;
	return false;
end
function ZXBCrossbowWeapon:hasBoltsInPouch(char)
	local boltPouch = char:getInventory():FindAndReturn('BoltQuiver')
	if not boltPouch then return false end
	return boltPouch:getInventory():FindAndReturn(self.ammoType) ~= nil
end
--************************************************************************--
--** ZXBCrossbowWeapon:reloadStart
--**
--** Function that should be performed upon the start of the timed action
--**
--** @param char - the character performing the action. Must not be nil
--** @param square - not used
--**
--************************************************************************--
function ZXBCrossbowWeapon:reloadStart(char, square, difficulty)
	self.reloadInProgress = true;
end

function ZXBCrossbowWeapon:reloadStart(char, square, difficulty)
	if not self:checkBreakCondition(char, char:getPrimaryHandItem()) then return false end
	return true;
end

--************************************************************************--
--** ZXBCrossbowWeapon:reloadPerform
--**
--** Function that should be performed upon successful completion of the
--** timed action
--**
--** @param char - the character performing the action. Must not be nil
--** @param square - not used
--** @param difficulty - the current reload difficulty
--** @param weapon - the item being reloaded
--**
--************************************************************************--
function ZXBCrossbowWeapon:reloadPerform(char, square, difficulty, weapon)
	getSoundManager():PlayWorldSound(self.insertSound, char:getSquare(), 0, 10, 1.0, false);
	local modData = weapon:getModData();		
	if modData.lastHitZedId and (modData.loadedAmmo == 'WoodenBolt' or modData.loadedAmmo == 'AluminumBolt') then
		if isClient() then
			print("Sending hit tracker update for Zed:"..modData.lastHitZedId);  				
		    sendClientCommand(char, "ZXB", "updateHitTracker", {zedId = modData.lastHitZedId, ammo =  modData.loadedAmmo});   
		else
			ZXBevents:updateHitTracker(modData.lastHitZedId, modData.loadedAmmo);
		end	
	end
	modData.lastHitZedId = nil

	self.currentCapacity = self.currentCapacity + 1;
	self.roundChambered = 1;
	-- remove the necessary ammo
	if self:hasBoltsInPouch(char) then
		if ZXBconfig.debug then print("Taking bolt from quiver") end
		char:getInventory():FindAndReturn('BoltQuiver'):getInventory():RemoveOneOf(self.ammoType)
	else
		char:getInventory():RemoveOneOf(self.ammoType);
	end
	
	-- We set a new ammo as default, execute hooks
	local wasAmmoSwitched = modData.loadedAmmo ~= self.ammoType;
	if  wasAmmoSwitched and ZXBAmmoTypeUtil[modData.loadedAmmo] then       
       ZXBAmmoTypeUtil[modData.loadedAmmo]['whenUnLoaded'](weapon);
    end
    modData.loadedAmmo = self.ammoType;
    if isClient() then
		sendClientCommand(char, "ZXB", "switchAmmo", {ammo =  self.ammoType}); 
	end  
	if wasAmmoSwitched and ZXBAmmoTypeUtil[modData.loadedAmmo] then
	    ZXBAmmoTypeUtil[modData.loadedAmmo]['whenLoaded'](weapon);
    end
	
	
	self.reloadInProgress = false;
	self:syncReloadableToItem(weapon);	
	char:getXp():AddXP(Perks.Reloading, 5);
	if(self.currentCapacity == self.maxCapacity) then
		return false;
	end
	return true;
end
function ZXBCrossbowWeapon:unloadPerform(char, square, difficulty, weapon)
	getSoundManager():PlayWorldSound(self.rackSound, char:getSquare(), 0, 10, 1.0, false);
	if self.roundChambered then
		self.roundChambered = 0;
	end	
	
	self.currentCapacity = self.currentCapacity - 1;
		
	
	-- Add the necessary ammo quiver first
	local quiver = char:getInventory():FindAndReturn('BoltQuiver');
	if quiver then--Add max check		
		quiver:getInventory():AddItem("ZXB."..self.ammoType);
	else		
		char:getInventory():AddItem("ZXB."..self.ammoType);
	end
	self.reloadInProgress = false;
	self:syncReloadableToItem(weapon);	
	if(self.currentCapacity == 0) then
		return false;
	end
	return true;
end

function ZXBCrossbowWeapon:checkBreakCondition(char, weapon)
	local inv = char:getInventory();
	local rand = ZombRand(1000);
	local woodworkingLevel = char:getPerkLevel(Perks.Woodwork);
	local reloadingLevel = char:getPerkLevel(Perks.Reloading);
	local breakChance = ZXBconfig.crossbowBreakChance - ((woodworkingLevel * 0.7) + (reloadingLevel * 0.7));
	if ZXBconfig.debug then
		print("Breakchance was: ".. breakChance.."Roll was: "..rand);
	end
	if rand < breakChance then		
		inv:Remove(weapon);
		char:Say("Oh Shit my "..weapon:getName());
		getSoundManager():PlayWorldSound('PZ_MetalSnap', char:getSquare(), 0, 10, 1.0, false);
		local brokenweapon = InventoryItemFactory.CreateItem('ZXB.'..weapon:getType()..'_Broken');
		brokenweapon:getModData().original = "ZXB."..weapon:getType();
		brokenweapon:getModData().loadedAmmo = weapon:getModData().loadedAmmo;
		brokenweapon:getModData().currentCapacity = self.currentCapacity;
		inv:AddItem(brokenweapon);
		char:setPrimaryHandItem(nil);			
	end
	return true;

end

function ZXBCrossbowWeapon:load(char, weapon, bolt)
	local sq = char:getSquare();	--maybe here
	if self.roundChambered == 1 then
	    self:unloadPerform(char, sq , 0, weapon);
    end
	weapon:getModData().defaultAmmo = bolt:getType();
	self.ammoType = bolt:getType();
	self:reloadPerform(char, sq, 0, weapon);
end
--************************************************************************--
--** ZXBCrossbowWeapon:rackingStart
--**
--** Function that should be performed upon the start of the timed action
--**
--** @param char - the character performing the action. Must not be nil
--** @param square - not used
--**
--************************************************************************--
function ZXBCrossbowWeapon:rackingStart(char, square, weapon)
    getSoundManager():PlayWorldSound(self.rackSound, char:getSquare(), 0, 10, 1.0, false);
	if self.emptyShellChambered == 1 and weapon:getShellFallSound() and weapon:getShellFallSound() ~= '' then
		getSoundManager():PlayWorldSound(weapon:getShellFallSound(), char:getSquare(), 0, 7, 1.0, false);
	end
end

--************************************************************************--
--** ZXBCrossbowWeapon:rackingPerform
--**
--** Function that should be performed upon successful completion of the
--** timed action
--**
--** @param char - the character performing the action. Must not be nil
--** @param square - not used
--** @param weapon - the item being reloaded
--**
--************************************************************************--
function ZXBCrossbowWeapon:rackingPerform(char, square, weapon)
	
    self.roundChambered = 0;
	self.emptyShellChambered = 0
    if(self.currentCapacity > 0) then
        self.currentCapacity = self.currentCapacity - 1;
        self.roundChambered = 1;
	end
	self:syncReloadableToItem(weapon);
end

--************************************************************************--
--** ZXBCrossbowWeapon:getReloadTime
--**
--** Returns the time take to perform the reload action
--**
--************************************************************************--
function ZXBCrossbowWeapon:getReloadTime()
	return self.reloadTime;
end

--************************************************************************--
--** ZXBCrossbowWeapon:getRackTime
--**
--** Returns the time take to perform the reload action
--**
--************************************************************************--
function ZXBCrossbowWeapon:getRackTime()
	return self.rackTime;
end

--************************************************************************--
--** ZXBCrossbowWeapon:syncItemToReloadable
--**
--** Function that copies details from an Item's modData to the instance
--** of this ZXBCrossbowWeapon
--**
--** @param weapon - the weapon from which the reloadable information
--** should be retrieved
--**
--************************************************************************--
function ZXBCrossbowWeapon:syncItemToReloadable(weapon)
	ISReloadableWeapon.syncItemToReloadable(self, weapon);
	local modData = weapon:getModData();
	self.roundChambered = modData.roundChambered;
	self.emptyShellChambered = modData.emptyShellChambered or 0
	self.ammoType = modData.defaultAmmo;
end

--************************************************************************--
--** ZXBCrossbowWeapon:syncReloadableToItem
--**
--** Function that copies details from the instance of this
--** ZXBCrossbowWeapon to an Item's modData
--**
--** @param weapon - the weapon to which the reloadable information
--** should be copied
--**
--************************************************************************--
function ZXBCrossbowWeapon:syncReloadableToItem(weapon)
	ISReloadableWeapon.syncReloadableToItem(self, weapon);
	local modData = weapon:getModData();
	modData.roundChambered = self.roundChambered;
	modData.emptyShellChambered = self.emptyShellChambered
end

--************************************************************************--
--** ZXBCrossbowWeapon:setupReloadable
--**
--** Function that initialises all the required modData on an item.
--**
--** @param weapon - the weapon to setup
--** @param v - the lua table containing the key value pairs to attach
--** to the modData
--************************************************************************--
function ZXBCrossbowWeapon:setupReloadable(weapon, v)
	ISReloadableWeapon.setupReloadable(self, weapon, v);
	local modData = weapon:getModData();
	modData.roundChambered = 0;
end

function ZXBCrossbowWeapon:printWeaponDetails(item)
    self:printItemDetails(item);
    local modData = item:getModData();
    local outString  = '';
    if(modData.roundChambered ~= nil) then
        outString = outString..', roundChambered: '..modData.roundChambered;
    else
        outString = outString..', roundChambered == nil';
    end
    if(modData.shootSoundPartial ~= nil) then
        outString = outString..', shootSoundPartial: '..modData.shootSoundPartial;
    else
        outString = outString..', shootSoundPartial == nil';
    end
    if(modData.defaultSwingSound ~= nil) then
        outString = outString..', defaultSwingSound: '..modData.defaultSwingSound;
    else
        outString = outString..', defaultSwingSound == nil';
    end
    print(outString);
    print('***************************************************************');
    print();
    print();
end


function ZXBCrossbowWeapon:printReloadableWeaponDetails()
    self:printReloadableDetails();
    local outString  = '';
    if(self.roundChambered ~= nil) then
        outString = outString..', roundChambered: '..self.roundChambered;
    else
        outString = outString..', roundChambered == nil';
    end
    if(self.containsClip ~= nil) then
        outString = outString..', shootSoundPartial: '..self.shootSoundPartial;
    else
        outString = outString..', shootSoundPartial == nil';
    end
    if(self.clipName ~= nil) then
        outString = outString..', defaultSwingSound: '..self.defaultSwingSound;
    else
        outString = outString..', defaultSwingSound == nil';
    end
    print(outString);
    print('***************************************************************');
    print();
    print();
end
